@@ -3,14 +3,27 @@
 /* Copies settings installed from gnome-initial-setup and
  * sticks them in the user's profile */
 
+#include <pwd.h>
+#include <string.h>
 #include <gio/gio.h>
-
-#define SKELETON_PATH "gnome-initial-setup/skeleton"
+#include <stdlib.h>
 
 static char *
-get_skeleton_dir (void)
+get_gnome_initial_setup_home_dir (void)
 {
-  return g_build_filename (g_get_user_runtime_dir (), SKELETON_PATH, NULL);
+  struct passwd pw, *pwp;
+  char buf[4096];
+
+  setpwent();
+  while (TRUE) {
+    if (getpwent_r (&pw, buf, sizeof (buf), &pwp))
+      break;
+
+    if (strcmp (pwp->pw_name, "gnome-initial-setup") == 0)
+      return g_strdup (pwp->pw_dir);
+  }
+
+  g_error ("Could not find homedir for gnome-initial-setup");
 }
 
 static void
@@ -20,11 +33,14 @@ move_file_from_tmpfs (GFile *src_base,
 {
   GFile *dest_dir = g_file_new_for_path (dir);
   GFile *dest = g_file_get_child (dest_dir, path);
+  GFile *dest_parent = g_file_get_parent (dest);
   gchar *basename = g_file_get_basename (dest);
   GFile *src = g_file_get_child (src_base, basename);
 
   GError *error = NULL;
 
+  g_file_make_directory_with_parents (dest_parent, NULL, NULL);
+
   if (!g_file_move (src, dest, G_FILE_COPY_NONE,
                     NULL, NULL, NULL, &error)) {
     if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND)) {
@@ -33,13 +49,7 @@ move_file_from_tmpfs (GFile *src_base,
                  g_file_get_path (dest),
                  error->message);
     }
-    g_error_free (error);
   }
-
-  g_object_unref (dest_dir);
-  g_object_unref (dest);
-  g_object_unref (src);
-  g_free (basename);
 }
 
 int
@@ -48,16 +58,13 @@ main (int    argc,
 {
   GFile *src;
   GError *error = NULL;
-  int ret = 0;
 
   g_type_init ();
 
-  src = g_file_new_for_path (get_skeleton_dir ());
+  src = g_file_new_for_path (get_gnome_initial_setup_home_dir ());
 
   if (!g_file_query_exists (src, NULL))
-    goto out;
-
-  ret = 1;
+    exit (EXIT_SUCCESS);
 
 #define FILE(d, x) \
   move_file_from_tmpfs (src, g_get_user_##d##_dir (), x)
@@ -65,17 +72,13 @@ main (int    argc,
   FILE (config, "run-welcome-tour");
   FILE (config, "dconf/user");
   FILE (config, "goa-1.0/accounts.conf");
-  FILE (data, "keyrings/Default.keyring");
+  FILE (data, "keyrings/login.keyring");
 
   if (!g_file_delete (src, NULL, &error))
     {
       g_warning ("Unable to delete skeleton dir: %s", error->message);
-      goto out;
+      exit (EXIT_FAILURE);
     }
 
-  ret = 0;
-
- out:
-  g_object_unref (src);
-  return ret;
+  return EXIT_SUCCESS;
 }
